@@ -26,6 +26,8 @@
 #include <telepathy-glib/enums.h>
 
 #include "empathy-contact-manager.h"
+#include "empathy-account-manager.h"
+#include "empathy-contact-monitor.h"
 #include "empathy-contact-list.h"
 #include "empathy-utils.h"
 
@@ -35,8 +37,8 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyContactManager)
 typedef struct {
 	GHashTable     *lists;
-	MissionControl *mc;
-	gpointer        token;
+	EmpathyAccountManager *account_manager;
+	EmpathyContactMonitor *contact_monitor;
 } EmpathyContactManagerPriv;
 
 static void contact_manager_iface_init         (EmpathyContactListIface    *iface);
@@ -159,23 +161,19 @@ contact_manager_add_account (EmpathyContactManager *manager,
 }
 
 static void
-contact_manager_status_changed_cb (MissionControl           *mc,
-				   TpConnectionStatus        status,
-				   McPresence                presence,
-				   TpConnectionStatusReason  reason,
-				   const gchar              *unique_name,
-				   EmpathyContactManager    *manager)
+contact_manager_connection_changed_cb (EmpathyAccountManager *account_manager,
+				       McAccount *account,
+				       TpConnectionStatusReason  reason,
+				       TpConnectionStatus current,
+				       TpConnectionStatus previous,
+				       EmpathyContactManager *manager)
 {
-	McAccount *account;
-
-	if (status != TP_CONNECTION_STATUS_CONNECTED) {
+	if (current != TP_CONNECTION_STATUS_CONNECTED) {
 		/* We only care about newly connected accounts */
 		return;
 	}
 
-	account = mc_account_lookup (unique_name);
 	contact_manager_add_account (manager, account);
-	g_object_unref (account);
 }
 
 static void
@@ -183,12 +181,19 @@ contact_manager_finalize (GObject *object)
 {
 	EmpathyContactManagerPriv *priv = GET_PRIV (object);
 
-	empathy_disconnect_account_status_changed (priv->token);
 	g_hash_table_foreach (priv->lists,
 			      contact_manager_disconnect_foreach,
 			      object);
 	g_hash_table_destroy (priv->lists);
-	g_object_unref (priv->mc);
+
+	g_signal_handlers_disconnect_by_func (priv->account_manager,
+					      contact_manager_connection_changed_cb,
+					      object);
+	g_object_unref (priv->account_manager);
+
+	if (priv->contact_monitor) {
+		g_object_unref (priv->contact_monitor);
+	}
 }
 
 static void
@@ -205,6 +210,7 @@ static void
 empathy_contact_manager_init (EmpathyContactManager *manager)
 {
 	GSList                    *accounts, *l;
+	MissionControl            *mc;
 	EmpathyContactManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
 		EMPATHY_TYPE_CONTACT_MANAGER, EmpathyContactManagerPriv);
 
@@ -213,19 +219,24 @@ empathy_contact_manager_init (EmpathyContactManager *manager)
 					     empathy_account_equal,
 					     (GDestroyNotify) g_object_unref,
 					     (GDestroyNotify) g_object_unref);
+	priv->account_manager = empathy_account_manager_dup_singleton ();
+	priv->contact_monitor = NULL;
 
-	priv->mc = empathy_mission_control_new ();
-	priv->token = empathy_connect_to_account_status_changed (priv->mc,
-		G_CALLBACK (contact_manager_status_changed_cb),
-		manager, NULL);
+	g_signal_connect (priv->account_manager,
+			  "account-connection-changed",
+			  G_CALLBACK (contact_manager_connection_changed_cb), manager);
+
+	mc = empathy_mission_control_new ();
 
 	/* Get ContactList for existing connections */
-	accounts = mission_control_get_online_connections (priv->mc, NULL);
+	accounts = mission_control_get_online_connections (mc, NULL);
 	for (l = accounts; l; l = l->next) {
 		contact_manager_add_account (manager, l->data);
 		g_object_unref (l->data);
 	}
+
 	g_slist_free (accounts);
+	g_object_unref (mc);
 }
 
 EmpathyContactManager *
@@ -319,6 +330,18 @@ contact_manager_get_members (EmpathyContactList *manager)
 	return contacts;
 }
 
+static EmpathyContactMonitor *
+contact_manager_get_monitor (EmpathyContactList *manager)
+{
+	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
+
+	if (priv->contact_monitor == NULL) {
+		priv->contact_monitor = empathy_contact_monitor_new_for_iface (manager);
+	}
+
+	return priv->contact_monitor;
+}
+
 static void
 contact_manager_get_pendings_foreach (McAccount             *account,
 				      EmpathyTpContactList  *list,
@@ -498,6 +521,7 @@ contact_manager_iface_init (EmpathyContactListIface *iface)
 	iface->add               = contact_manager_add;
 	iface->remove            = contact_manager_remove;
 	iface->get_members       = contact_manager_get_members;
+	iface->get_monitor       = contact_manager_get_monitor;
 	iface->get_pendings      = contact_manager_get_pendings;
 	iface->get_all_groups    = contact_manager_get_all_groups;
 	iface->get_groups        = contact_manager_get_groups;
@@ -507,3 +531,19 @@ contact_manager_iface_init (EmpathyContactListIface *iface)
 	iface->remove_group	 = contact_manager_remove_group;
 }
 
+gboolean
+empathy_contact_manager_can_add (EmpathyContactManager *manager,
+				 McAccount             *account)
+{
+	EmpathyContactManagerPriv *priv = GET_PRIV (manager);
+	EmpathyTpContactList      *list;
+	
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_MANAGER (manager), FALSE);
+
+	list = g_hash_table_lookup (priv->lists, account);
+	if (list == NULL)
+		return FALSE;
+
+	return empathy_tp_contact_list_can_add (list);
+}
+
