@@ -23,7 +23,7 @@
 #include <config.h>
 
 #include <string.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <telepathy-glib/channel.h>
 #include <telepathy-glib/connection.h>
@@ -843,8 +843,11 @@ tp_contact_list_add (EmpathyContactList *list,
 
 	g_return_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list));
 
-	empathy_tp_group_add_member (priv->subscribe, contact, message);
-	if (g_list_find (priv->pendings, contact)) {
+	if (priv->subscribe) {
+		empathy_tp_group_add_member (priv->subscribe, contact, message);
+	}
+
+	if (priv->publish && g_list_find (priv->pendings, contact)) {
 		empathy_tp_group_add_member (priv->publish, contact, message);		
 	}
 }
@@ -858,8 +861,12 @@ tp_contact_list_remove (EmpathyContactList *list,
 
 	g_return_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list));
 
-	empathy_tp_group_remove_member (priv->subscribe, contact, message);
-	empathy_tp_group_remove_member (priv->publish, contact, message);		
+	if (priv->subscribe) {
+		empathy_tp_group_remove_member (priv->subscribe, contact, message);
+	}
+	if (priv->publish) {
+		empathy_tp_group_remove_member (priv->publish, contact, message);		
+	}
 }
 
 static GList *
@@ -1000,7 +1007,9 @@ tp_contact_list_add_to_group (EmpathyContactList *list,
 	tp_group = tp_contact_list_get_group (EMPATHY_TP_CONTACT_LIST (list),
 					      group);
 
-	empathy_tp_group_add_member (tp_group, contact, "");
+	if (tp_group) {
+		empathy_tp_group_add_member (tp_group, contact, "");
+	}
 }
 
 static void
@@ -1094,3 +1103,19 @@ tp_contact_list_iface_init (EmpathyContactListIface *iface)
 	iface->remove_group	 = tp_contact_list_remove_group;
 }
 
+gboolean
+empathy_tp_contact_list_can_add (EmpathyTpContactList *list)
+{
+	EmpathyTpContactListPriv *priv;
+	TpChannelGroupFlags       flags;
+
+	g_return_val_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list), FALSE);
+
+	priv = GET_PRIV (list);
+
+	if (priv->subscribe == NULL)
+		return FALSE;
+
+	flags = empathy_tp_group_get_flags (priv->subscribe);
+	return (flags & TP_CHANNEL_GROUP_FLAG_CAN_ADD) != 0;
+}
