@@ -30,6 +30,7 @@
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-tp-call.h>
+#include <libempathy/empathy-tp-group.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
@@ -193,6 +194,9 @@ static void
 call_window_hang_up_button_clicked_cb (GtkWidget *widget,
                                        EmpathyCallWindow *window)
 {
+  empathy_sound_play (GTK_WIDGET (window->window),
+		      EMPATHY_SOUND_PHONE_HANGUP);
+
   DEBUG ("Call clicked, end call");
   call_window_finalize (window);
 }
@@ -359,6 +363,15 @@ call_window_update (EmpathyCallWindow *window)
       "is-incoming: %d video-stream direction: %d",
       window->status, stream_state, is_incoming, video_stream->direction);
 
+  if (empathy_tp_call_has_dtmf (window->call))
+    {
+      gtk_widget_show (window->keypad_expander);
+    }
+  else
+    {
+      gtk_widget_hide (window->keypad_expander);
+    }
+
   /* Depending on the status we have to set:
    * - window's title
    * - status's label
@@ -367,6 +380,8 @@ call_window_update (EmpathyCallWindow *window)
   if (window->status == EMPATHY_TP_CALL_STATUS_READYING)
     {
       gtk_window_set_title (GTK_WINDOW (window->window), _("Empathy Call"));
+      /* To translators: Readying is the first state of the call, it is
+       * preparing the connection and it does not yet ring. */
       gtk_label_set_text (GTK_LABEL (window->status_label), _("Readying"));
       gtk_widget_set_sensitive (window->video_button, FALSE);
       gtk_widget_set_sensitive (window->output_volume_button, FALSE);
@@ -382,8 +397,18 @@ call_window_update (EmpathyCallWindow *window)
       gtk_window_set_title (GTK_WINDOW (window->window), title);
       gtk_label_set_text (GTK_LABEL (window->status_label), _("Ringing"));
       gtk_widget_set_sensitive (window->hang_up_button, TRUE);
+
       if (is_incoming)
+        {
           call_window_show_confirmation_dialog (window);
+          empathy_sound_play (GTK_WIDGET (window->window),
+			      EMPATHY_SOUND_PHONE_INCOMING);
+        }
+      else
+        {
+          empathy_sound_play (GTK_WIDGET (window->window),
+			      EMPATHY_SOUND_PHONE_OUTGOING);
+	}
     }
   else if (window->status == EMPATHY_TP_CALL_STATUS_ACCEPTED)
     {
@@ -484,10 +509,24 @@ empathy_call_window_new (TpChannel *channel)
           g_signal_connect_swapped (G_OBJECT (window->call), "notify",
               G_CALLBACK (call_window_update), window);
           call_window_update (window);
+
+          if (GTK_WIDGET_REALIZED (GTK_WIDGET (window->preview_video_socket)))
+            {
+              call_window_socket_realized_cb (window->preview_video_socket,
+                  window);
+            }
         }
       else
         {
           GtkWidget *dialog;
+          EmpathyContact *contact;
+          EmpathyTpGroup *tp_group;
+
+          tp_group = empathy_tp_group_new (channel);
+          empathy_run_until_ready (tp_group);
+          empathy_tp_group_get_invitation (tp_group, &contact);
+          empathy_contact_run_until_ready (contact, EMPATHY_CONTACT_READY_NAME,
+              NULL);
 
           /* We don't want to have multiple calls running.
            * FIXME: We should use the hold interface... */
@@ -495,7 +534,12 @@ empathy_call_window_new (TpChannel *channel)
 
           dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
               GTK_BUTTONS_CLOSE,
-              _("Call rejected because there is already a running call."));
+              _("Incoming call from %s rejected because there is already a"
+                " running call."), empathy_contact_get_name (contact));
+
+          g_object_unref (contact);
+          g_object_unref (tp_group);
+
           g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy),
               NULL);
           gtk_widget_show (dialog);
